// 
//  ode.java written by Kenji Shimada
//  Modified by Arbtip Dheeravongkit
//  Problem Set 12 Sample Code 
// 

import java.io.*;
import java.math.*;

public class Ode {

public static void main(String args[]) throws IOException {     

	double[][] q = new double[N][6];	// matrix to store the state of the system
	double[] f = new double[6];		// dq/dt = f
	double	l, c;		// variables for intermediate results
	int i, j;		// loop counters


	System.out.println("\n>> 24-311:Numerical Methods--ODE <<\n\n");

	//  Check the number of arguments 
	if (args.length == 6) {
		//Set an initial location and velocity
		for(i=0; i<6; i++) {
		    Double I = new Double(args[i]);
		    q[0][i] = I.doubleValue();
		}     
	} else {
		System.out.println("usage: \"ode x0 y0 z0 vx0 vy0 vz0\" \n");
		System.exit(-1); 
	}


	//  Open output wrl file
	outFile = new PrintWriter (new FileWriter ("output.wrl"));
	myName = "Kenji Shimada (shimada@cmu.edu)";

	/////////////////////////////////////////////////////////////////
	//The following sample code integrates the equation of motion
	//using the Euler's method.  You are asked to replace it with 
	//the fourth order Runge-Kutta method.
	/////////////////////////////////////////////////////////////////
	for (i=0; i<N-1; i++){
		l = Math.sqrt( q[i][0]*q[i][0] + q[i][1]*q[i][1] + q[i][2]*q[i][2] );
		c = - K * (l - LO) / (l*M);
		f[0] = q[i][3];
		f[1] = q[i][4];
		f[2] = q[i][5];
		f[3] = c * q[i][0];
		f[4] = c * q[i][1] - G;
		f[5] = c * q[i][2];
		for (j=0; j<6; j++)
			q[i+1][j] = q[i][j] + H*f[j];
		System.out.print(q[i][0]);
		System.out.print("  ");
		System.out.print(q[i][1]);
		System.out.print("  ");
		System.out.print(q[i][2]);
		System.out.print("\n");
		System.out.println();
	}


	//  Create output wrl file
	vrml_ode (q);
	
	//  Close the output wrl file
	outFile.close();

}

public static void vrml_ode (double q[][]) {
//  outFile: pointer to the output vrml file
//  my_name: my name to be shown in the heading
//	q[][6]:  matrix to store a 3D trajectory
	
	int		i;

	// Create Header
	outFile.print( "#VRML V2.0 utf8\n");
	outFile.print( "#------------------------------------------------\n");
	outFile.print( "# 24-311: Numerical Methods  Fall'98\n");
	outFile.print( "# ");
	outFile.print(myName);
	outFile.print("\n");
	outFile.print( "# ODE\n");
	outFile.print( "#------------------------------------------------\n\n");

	// Set up background
	outFile.print( "# Specify background color\n");
	outFile.print( "#-----------------------------------\n");
	outFile.print( "Background {\n\tskyColor 1 1 1\n}\n\n"); // White background

	// Create the ceiling
	outFile.print( "# Draw canvas\n");
	outFile.print( "#-----------------------------------\n");
	outFile.print( "Transform {\n");
	outFile.print( "\trotation 1 0 0 1.5707\n");
	outFile.print( "\ttranslation 0 0 0\n");
	outFile.print( "\tchildren Shape {\n");
	outFile.print( "\t\tappearance Appearance {\n");
	outFile.print( "\t\t\tmaterial Material {\n");
	outFile.print( "\t\t\t\tdiffuseColor 0.0 0.9 0.9\n");
	outFile.print( "\t\t\t\ttransparency 0.5\n\t\t\t}\n\t\t}\n");
	outFile.print( "\t\tgeometry Box {\n");
	outFile.print( "\t\t\tsize 4 4 0.05\n\t\t}\n\t}\n}\n\n");

	// Put text heading on the VRML screen
	outFile.print( "# Draw texts\n");
	outFile.print( "#-----------------------------------\n");
	outFile.print( "Transform {\n");
	outFile.print( "\ttranslation 0 2 0.2\n");
	outFile.print( "\tchildren Shape {\n");
	outFile.print( "\t\tappearance Appearance {\n");
	outFile.print( "\t\t\tmaterial Material {\n");
	outFile.print( "\t\t\t\tdiffuseColor 1 0 0\n");
	outFile.print( "\t\t\t}\n\t\t}\n");
	outFile.print( "\t\tgeometry Text {\n");
	outFile.print( "\t\t\tstring [\"24-311: Numerical Methods--PS11\",\n");
	outFile.print( "\t\t\t\"Ordinary Differential Equation\",\n");
	outFile.print( "\t\t\t\"");
	outFile.print( myName);
	outFile.print("\"]\n");
	outFile.print( "\t\t\tfontStyle FontStyle {\n");
	outFile.print( "\t\t\t\tfamily \"SANS\"\n");
	outFile.print( "\t\t\t\tjustify \"MIDDLE\"\n");
	outFile.print( "\t\t\t\tstyle \"BOLD\"\n");
	outFile.print( "\t\t\t\tsize 0.3\n\t\t\t}\n\t\t}\n\t}\n}\n\n");

	// Add x-axis
	outFile.print( "# Define x axis\n");
	outFile.print( "#-----------------------------------\n");
	outFile.print( "Transform {\n");
	outFile.print( "\ttranslation 0 0 0\n");
	outFile.print( "\tchildren DEF AXIS Shape {\n");
	outFile.print( "\t\tappearance Appearance {\n");
	outFile.print( "\t\t\tmaterial Material {\n");
	outFile.print( "\t\t\t\tdiffuseColor 0.0 0.0 0.0\n\t\t\t}\n\t\t}\n");
	outFile.print( "\t\tgeometry Extrusion {\n");
	outFile.print( "\t\t\tcrossSection [1 0, 0 -1, -1 0, 0 1, 1 0]\n");
	outFile.print( "\t\t\tscale [0.01 0.01, 0.01 0.01, 0.04 0.04, 0 0]\n");
	outFile.print( "\t\t\tspine [0 0 0, 2.1 0 0, 2.1 0 0, 2.2 0 0]\n\t\t}\n\t}\n}\n\n");

	// Add y-axis
	outFile.print( "# Draw y axis\n");
	outFile.print( "#-----------------------------------\n");
	outFile.print( "Transform {\n");
	outFile.print( "\trotation 0 0 1 1.5707\n");
	outFile.print( "\tchildren USE AXIS\n}\n\n");

	// Add z-axis
	outFile.print( "# Draw z axis\n");
	outFile.print( "#-----------------------------------\n");
	outFile.print( "Transform {\n");
	outFile.print( "\trotation 0 1 0 -1.5707\n");
	outFile.print( "\tchildren USE AXIS\n}\n\n");

	// Draw mass
	outFile.print( "# Draw mass\n");
	outFile.print( "#-----------------------------------\n");
	outFile.print( "DEF T Transform {\n");
	outFile.print( "\ttranslation 0 -3 0\n");
	outFile.print( "\tchildren [\n");
	outFile.print( "\t\tShape {\n");
	outFile.print( "\t\t\tgeometry Box { size 0.2 0.2 0.2 }\n");
	outFile.print( "\t\t\tappearance Appearance {\n");
    outFile.print( "\t\t\t\tmaterial Material {\n");
	outFile.print( "\t\t\t\t\tdiffuseColor 1 0 0\n");
	outFile.print( "\t\t\t\t}\n");
	outFile.print( "\t\t\t}\n");
	outFile.print( "\t\t}\n");
	outFile.print( "\t]\n");
	outFile.print( "}\n\n");

	outFile.print( "DEF P PositionInterpolator {\n");
    outFile.print( "\tkey [");
	for(i=0; i<N; i++){
		outFile.print( ((double) i) / ((double) N));
		outFile.print(", ");
	}
	outFile.print( "]\n");
    outFile.print( "\tkeyValue [");
	for(i=0; i<N; i++){
		outFile.print( q[i][0]);
		outFile.print(" ");
		outFile.print(q[i][1]); 
		outFile.print(" ");
		outFile.print(q[i][2]);
		outFile.print(", ");
	}
	outFile.print( "]\n");
	outFile.print( "}\n\n");

	outFile.print( "DEF TS TimeSensor {\n");
	outFile.print( "\tcycleInterval ");
	outFile.print(N*H); 
	outFile.print("\n");
	outFile.print( "\tloop     TRUE\n");
	outFile.print( "}\n\n");

	outFile.print( "ROUTE TS.fraction_changed TO P.set_fraction\n");
	outFile.print( "ROUTE P.value_changed TO T.set_translation\n\n");

	// Draw trajectory
	outFile.print( "# Draw trajectory\n");
	outFile.print( "#-----------------------------------\n");
	outFile.print( "Transform {\n");
	outFile.print( "\tchildren Shape {\n");
	outFile.print( "\t\tappearance Appearance {\n");
	outFile.print( "\t\t\tmaterial Material {\n");
	outFile.print( "\t\t\t\tdiffuseColor 0.0 1.0 0.0\n");
	outFile.print( "\t\t\t}\n");
	outFile.print( "\t\t}\n");
	outFile.print( "\t\tgeometry IndexedLineSet {\n");
	outFile.print( "\t\t\tcoord Coordinate {\n");
	outFile.print( "\t\t\t\tpoint [");
	for(i=0; i<N; i++){
		outFile.print( q[i][0] );
		outFile.print(" ");
		outFile.print(q[i][1]);
		outFile.print(" ");
		outFile.print(q[i][2]);
		outFile.print(", ");
	}
	outFile.print( "]\n");
	outFile.print( "\t\t\t}\n");
	outFile.print( "\t\t\tcoordIndex [");
	for(i=0; i<N; i++){
		outFile.print( i);
		outFile.print( ", ");
	}
	outFile.print( "]\n");
	outFile.print( "\t\t}\n");
	outFile.print( "\t}\n");
	outFile.print( "}\n");

}

    //class variables
    private static PrintWriter outFile;
    private static String myName;

  //Constants
  private static final double LO = 2.0;
  private static final int N = 1000;
  private static final double H = 0.01;
  private static final double K = 10.0;
  private static final double G = 9.8;
  private static final double M = 1.0;

}





